@@ -5,9 +5,13 @@
 import sys
 
 import unittest
+import tempfile
+import shutil
+import os
+import locale
+import subprocess
 from gi.repository import GObject
 
-import gobject
 from gi.repository import GIMarshallingTests
 
 from compathelper import _bytes
@@ -737,7 +741,6 @@ class TestArray(unittest.TestCase):
     def test_gstrv_inout(self):
         self.assertEquals(['-1', '0', '1', '2'], GIMarshallingTests.gstrv_inout(['0', '1', '2']))
 
-
 class TestGArray(unittest.TestCase):
 
     def test_garray_int_none_return(self):
@@ -781,6 +784,37 @@ class TestGArray(unittest.TestCase):
     def test_garray_utf8_full_inout(self):
         self.assertEquals(['-2', '-1','0', '1'], GIMarshallingTests.garray_utf8_full_inout(['0', '1', '2']))
 
+class TestGPtrArray(unittest.TestCase):
+
+    def test_gptrarray_utf8_none_return(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_none_return())
+
+    def test_gptrarray_utf8_container_return(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_container_return())
+
+    def test_gptrarray_utf8_full_return(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_full_return())
+
+    def test_gptrarray_utf8_none_in(self):
+        GIMarshallingTests.gptrarray_utf8_none_in(Sequence(['0', '1', '2']))
+
+    def test_gptrarray_utf8_none_out(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_none_out())
+
+    def test_gptrarray_utf8_container_out(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_container_out())
+
+    def test_gptrarray_utf8_full_out(self):
+        self.assertEquals(['0', '1', '2'], GIMarshallingTests.gptrarray_utf8_full_out())
+
+    def test_gptrarray_utf8_none_inout(self):
+        self.assertEquals(['-2', '-1', '0', '1'], GIMarshallingTests.gptrarray_utf8_none_inout(Sequence(('0', '1', '2'))))
+
+    def test_gptrarray_utf8_container_inout(self):
+        self.assertEquals(['-2', '-1','0', '1'], GIMarshallingTests.gptrarray_utf8_container_inout(['0', '1', '2']))
+
+    def test_gptrarray_utf8_full_inout(self):
+        self.assertEquals(['-2', '-1','0', '1'], GIMarshallingTests.gptrarray_utf8_full_inout(['0', '1', '2']))
 
 class TestGList(unittest.TestCase):
 
@@ -945,6 +979,10 @@ class TestGClosure(unittest.TestCase):
     def test_gclosure_in(self):
         GIMarshallingTests.gclosure_in(lambda: 42)
 
+        # test passing a closure between two C calls
+        closure = GIMarshallingTests.gclosure_return()
+        GIMarshallingTests.gclosure_in(closure)
+
         self.assertRaises(TypeError, GIMarshallingTests.gclosure_in, 42)
         self.assertRaises(TypeError, GIMarshallingTests.gclosure_in, None)
 
@@ -956,6 +994,31 @@ class TestPointer(unittest.TestCase):
 
 class TestEnum(unittest.TestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        '''Run tests under a test locale.
+
+        Upper case conversion of member names should not be locale specific;
+        e.  g. in Turkish, "i".upper() == "i", which gives results like "iNVALiD"
+
+        Run test under a locale which defines toupper('a') == 'a'
+        '''
+        cls.locale_dir = tempfile.mkdtemp()
+        subprocess.check_call(['localedef', '-i',
+            os.path.join(os.path.dirname(os.path.realpath(__file__)), 'te_ST@nouppera'),
+            '-c', '-f', 'UTF-8', os.path.join(cls.locale_dir, 'te_ST.UTF-8@nouppera')])
+        os.environ['LOCPATH'] = cls.locale_dir
+        locale.setlocale(locale.LC_ALL, 'te_ST.UTF-8@nouppera')
+
+    @classmethod
+    def tearDownClass(cls):
+        locale.setlocale(locale.LC_ALL, 'C')
+        shutil.rmtree(cls.locale_dir)
+        try:
+            del os.environ['LOCPATH']
+        except KeyError:
+            pass
+
     def test_enum(self):
         self.assertTrue(issubclass(GIMarshallingTests.Enum, int))
         self.assertTrue(isinstance(GIMarshallingTests.Enum.VALUE1, GIMarshallingTests.Enum))
@@ -1512,7 +1575,8 @@ class TestPythonGObject(unittest.TestCase):
         # compare the same enum from both the pygobject attrs and gi GObject attrs
         self.assertEquals(GObject.SIGNAL_ACTION, GObject.SignalFlags.ACTION)
         # compare a static gobject attr with a dynamic GObject attr
-        self.assertEquals(GObject.GObject, gobject.GObject)
+        import gi._gobject
+        self.assertEquals(GObject.GObject, gi._gobject.GObject)
 
     def test_subobject_non_vfunc_do_method(self):
         class PythonObjectWithNonVFuncDoMethod:
@@ -1716,3 +1780,61 @@ class TestGErrorArrayInCrash(unittest.TestCase):
     # take in GArrays. See https://bugzilla.gnome.org/show_bug.cgi?id=642708
     def test_gerror_array_in_crash(self):
         self.assertRaises(GObject.GError, GIMarshallingTests.gerror_array_in, [1, 2, 3])
+
+class TestKeywordArgs(unittest.TestCase):
+    def test_calling(self):
+        kw_func = GIMarshallingTests.int_three_in_three_out
+
+        self.assertEquals(kw_func(1, 2, 3),                 (1, 2, 3))
+        self.assertEquals(kw_func(**{'a':4, 'b':5, 'c':6}), (4, 5, 6))
+        self.assertEquals(kw_func(1, **{'b':7, 'c':8}),     (1, 7, 8))
+        self.assertEquals(kw_func(1, 7, **{'c':8}),         (1, 7, 8))
+        self.assertEquals(kw_func(1, c=8, **{'b':7}),       (1, 7, 8))
+        self.assertEquals(kw_func(2, c=4, b=3),             (2, 3, 4))
+        self.assertEquals(kw_func(a=2, c=4, b=3),           (2, 3, 4))
+
+    def assertRaisesMessage(self, exception, message, func, *args, **kwargs):
+        try:
+            func(*args, **kwargs)
+        except exception:
+            (e_type, e) = sys.exc_info()[:2]
+            if message is not None:
+                self.assertEqual(str(e), message)
+        except:
+            raise
+        else:
+            msg = "%s() did not raise %s" % (func.__name__, exception.__name__)
+            raise AssertionError(msg)
+
+    def test_type_errors(self):
+        # test too few args
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 arguments (0 given)",
+                                 GIMarshallingTests.int_three_in_three_out)
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 arguments (1 given)",
+                                 GIMarshallingTests.int_three_in_three_out, 1)
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 arguments (0 given)",
+                                 GIMarshallingTests.int_three_in_three_out, *())
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 arguments (0 given)",
+                                 GIMarshallingTests.int_three_in_three_out, *(), **{})
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 non-keyword arguments (0 given)",
+                                 GIMarshallingTests.int_three_in_three_out, *(), **{'c':4})
+
+        # test too many args
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 arguments (4 given)",
+                                 GIMarshallingTests.int_three_in_three_out, *(1, 2, 3, 4))
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() takes exactly 3 non-keyword arguments (4 given)",
+                                 GIMarshallingTests.int_three_in_three_out, *(1, 2, 3, 4), c=6)
+
+        # test too many keyword args
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() got multiple values for keyword argument 'a'",
+                                 GIMarshallingTests.int_three_in_three_out, 1, 2, 3, **{'a': 4, 'b': 5})
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() got an unexpected keyword argument 'd'",
+                                 GIMarshallingTests.int_three_in_three_out, d=4)
+        self.assertRaisesMessage(TypeError, "int_three_in_three_out() got an unexpected keyword argument 'e'",
+                                 GIMarshallingTests.int_three_in_three_out, **{'e': 2})
+
+    def test_kwargs_are_not_modified(self):
+        d = {'b': 2}
+        d2 = d.copy()
+        GIMarshallingTests.int_three_in_three_out(1, c=4, **d)
+        self.assertEqual(d, d2)
